/*
Copyright (c) 2001 The University of Texas
All Rights Reserved.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

The GNU Public License is available in the file LICENSE, or you
can write to the Free Software Foundation, Inc., 59 Temple Place -
Suite 330, Boston, MA 02111-1307, USA, or you can find it on the
World Wide Web at http://www.fsf.org.

Programmers:    Wade Schwartzkopf
Version:        @(#)classify.h    1.0     08/16/02

The author performed this work while he was with the Department of
Electrical and Computer Engineering, The University of Texas at Austin,
Austin, TX.  Wade Schwartzkopf can be reached at wade@schwartzkopf.org.
The professors supervising the work were Prof. Alan C. Bovik,
bovik@ece.utexas.edu, and Prof. Brian L. Evans, bevans@ece.utexas.edu.
*/

#include "mfish.h"

struct trainDataStruct {
   float** means;
   float*** covarianceInverses;
   float* covarianceDeterminants;
   float* priors;
   float* sizeMeans;
   float* sizeVariances;

   int classes;
   int spectra;
};

// Allocate memory for training data data structure
int initTrainingDataStruct(struct trainDataStruct* trainData, int class, int spectra);

// Free memory allocated for training data data structure
int freeTrainingDataStruct(struct trainDataStruct* trainData);

// Read training data from file (also allocates memory for data structure)
int readTrainingData(struct trainDataStruct* trainData, char* MFISHdirectory, char* imageName);

// Calculate point in multi-dimensional gaussian
float calcGaussian(float* x, float* mean, float** invcov, float covdet, int dimensions);

// Calculate probability of each pixel belonging to each class
int calcProbabilities(MFISH mfImage, unsigned char** mask, struct trainDataStruct* trainData, float*** probs, int rows, int cols);

// Select most likely class for each pixel
int classifyPixels(float*** probs, unsigned char** mask, unsigned char** outputImage, int rows, int cols, int spectra);
