/*
Copyright (c) 2001 The University of Texas
All Rights Reserved.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

The GNU Public License is available in the file LICENSE, or you
can write to the Free Software Foundation, Inc., 59 Temple Place -
Suite 330, Boston, MA 02111-1307, USA, or you can find it on the
World Wide Web at http://www.fsf.org.

Programmers:    Wade Schwartzkopf
Version:        @(#)imgtools.c    2.0     08/16/00

The author performed this work while he was with the Department of
Electrical and Computer Engineering, The University of Texas at Austin,
Austin, TX.  Wade Schwartzkopf can be reached at wade@schwartzkopf.org.
The professors supervising the work were Prof. Alan C. Bovik,
bovik@ece.utexas.edu, and Prof. Brian L. Evans, bevans@ece.utexas.edu.
*/

#include <math.h>
#include <stdlib.h>

#include "imgtools.h"
#include "my_matrix_stuff.h"
#include "miscfunc.h"

void threshold(unsigned char** inputImage, unsigned char** outputImage, unsigned char thresh, int rows, int cols)
{
   int i,j;

   for(i = 0; i < rows; i++) {
      for(j = 0; j < cols; j++) {
         outputImage[i][j]=((inputImage[i][j])>thresh);
      }
   }
}

int findArea(unsigned char** mask, int lowX, int highX, int lowY, int highY)
{
   int i,j;
   int area=0;

   for(i = lowY; i <= highY; i++) {
      for(j = lowX; j <= highX; j++) {
         if(mask[i][j]) { area++; }
      }
   }

   return area;
}

void multiplyImage(unsigned char** inputImage, unsigned char** outputImage, int factor, int rows, int cols)
{
   int i,j;

   for(i = 0; i < rows; i++) {
      for(j = 0; j < cols; j++) {  
         outputImage[i][j]=(inputImage[i][j])*factor;
      }
   }
}

void notImage(unsigned char** inputImage, unsigned char** outputImage, int lowX, int highX, int lowY, int highY)
{
   int i,j;

   for(i = lowY; i <= highY; i++) {
      for(j = lowX; j <= highX; j++) {
         outputImage[i][j]=!(inputImage[i][j]);
      }
   }
}

void orImage(unsigned char** inputImage1, unsigned char** inputImage2, unsigned char** outputImage, int lowX, int highX, int lowY, int highY)
{
   int i,j;

   for(i = lowY; i <= highY; i++) {
      for(j = lowX; j <= highX; j++) {
         outputImage[i][j]=(inputImage1[i][j])||(inputImage2[i][j]);
      }
   }
}

void clearImage(unsigned char** image, int lowX, int highX, int lowY, int highY)
{
   int i,j;

   for(i = lowY; i <= highY; i++) {
      for(j = lowX; j <= highX; j++) {
        image[i][j]=0;
      }
   }
}

void relabel(unsigned char** inputImage, unsigned char** outputImage, int oldClass, int newClass, int lowX, int highX, int lowY, int highY)
{
   int i,j;

   for(i = lowY; i <= highY; i++) {
      for(j = lowX; j <= highX; j++) {
         if((inputImage[i][j])==oldClass)
            outputImage[i][j]=newClass;
      }
   }
}

void skiplabel(unsigned char** mask, int label2, int lowX, int highX, int lowY, int highY)
{
   int i,j;

   for(i = lowY; i <= highY; i++) {
      for(j = lowX; j <= highX; j++) {
         if((mask[i][j])>label2) { (mask[i][j])--; }
      }
   }
}

int max(unsigned char** image, int lowX, int highX, int lowY, int highY)
{
   int i,j;
   unsigned char currentMax = image[lowY][lowX];

   for(i = lowY; i <= highY; i++) {
      for(j = lowX; j <= highX; j++) {
         if(image[i][j]>currentMax) {
            currentMax=image[i][j];
         }
      }
   }

   return currentMax;
}

void find_equal(unsigned char** inputImage, unsigned char** outputImage, unsigned char valueToFind, int lowX, int highX, int lowY, int highY)
{
   int i,j;

   for(i = lowY; i <= highY; i++) {
      for(j = lowX; j <= highX; j++) {
         outputImage[i][j] = (inputImage[i][j]==valueToFind);
      }
   }
}

void find_two_equal(unsigned char** inputImage, unsigned char** outputImage, unsigned char valueToFind1, unsigned char valueToFind2, int lowX, int highX, int lowY, int highY)
{
   int i,j;

   for(i = lowY; i <= highY; i++) {
      for(j = lowX; j <= highX; j++) {
         outputImage[i][j] = (inputImage[i][j]==valueToFind1) ||
                             (inputImage[i][j]==valueToFind2);
      }
   }
}

void ifthen(unsigned char** ifImage, unsigned char** thenImage, unsigned char** outputImage, int rows, int cols)
{
   int i,j;

   for(i = 0; i < rows; i++) {
      for(j = 0; j < cols; j++) {
         outputImage[i][j] = thenImage[i][j]*((ifImage[i][j])>0);
      }
   }
}

void boundingBox(unsigned char** mask, int* lowX, int* highX, int* lowY, int* highY, int rows, int cols)
{
   int i,j;
   
   *lowX=cols;
   *highX=0;
   *lowY=rows;
   *highY=0;
   for(i=0;i<rows;i++) {
      for(j=0;j<cols;j++) {
         if(mask[i][j]) {
            if(i<*lowY) { *lowY=i; }
            if(i>*highY) { *highY=i; }
            if(j<*lowX) { *lowX=j; }
            if(j>*highX) { *highX=j; }
         }
      }
   }
}

void findHoles(unsigned char** inputImage, unsigned char** holes, int lowX, int highX, int lowY, int highY)
{
   int i;
   unsigned char** temp=allocateByteMatrix(highY+1, highX+1);

   notImage(inputImage, temp, lowX, highX, lowY, highY);
   for(i = lowX; i<=highX; i++) {
      temp[lowY][i]=1;
      temp[highY][i]=1;
   }
   for(i = lowY; i<=highY; i++) {
      temp[i][lowX]=1;
      temp[i][highX]=1;
   }
   labelObjects(temp, temp, holes, 1, lowX, highX, lowY, highY, 8);
   find_equal(holes, holes, 1, lowX, highX, lowY, highY);
   orImage(holes, inputImage, holes, lowX, highX, lowY, highY);
   notImage(holes, holes, lowX, highX, lowY, highY);

   freeByteMatrix(temp);
}

int labelSingleObject(unsigned char** inputImage, unsigned char** mask, unsigned char** outputImage, int startX, int startY, unsigned char class, int backFore, int lowX, int highX, int lowY, int highY, int connectedness)
{
   const int connectedX[] = X_NEIGHBORS;
   const int connectedY[] = Y_NEIGHBORS;
   int i, pixelQueueSize=0;
   int x, y;
   int pixelQueue[(1+highY-lowY)*(1+highX-lowX)];
   int propogationConditon;
   int area=0;

   outputImage[startY][startX]=class;
   pixelQueue[0] = (startY << 16) + startX;
   pixelQueueSize = 1;

   while(pixelQueueSize > 0) {
      x=pixelQueue[0] & 0xffff;
      y=(pixelQueue[0] >> 16) & 0xffff;
      pixelQueue[0]=pixelQueue[--pixelQueueSize];
      for(i = 0; i < connectedness; i++) {
         if (((x+connectedX[i])>=lowX)&&((x+connectedX[i])<=highX)&&
             ((y+connectedY[i])>=lowY)&&((y+connectedY[i])<=highY)) {
            if (backFore) {
               propogationConditon=(inputImage[y+connectedY[i]][x+connectedX[i]]>0);
            }
            else {
               propogationConditon=(inputImage[y+connectedY[i]][x+connectedX[i]]==inputImage[startY][startX]);
            }
            if (propogationConditon&&mask[y+connectedY[i]][x+connectedX[i]]&&
                (outputImage[y+connectedY[i]][x+connectedX[i]]!=class)) {
               outputImage[y+connectedY[i]][x+connectedX[i]]=class;
               area++;
               pixelQueue[pixelQueueSize++] = ((y+connectedY[i]) << 16) + (x+connectedX[i]);
            }
         }
      }
   }

   return area;
}

int labelObjects(unsigned char** inputImage, unsigned char** mask, unsigned char** outputImage, int backFore, int lowX, int highX, int lowY, int highY, int connectedness)
{
   int i,j;
   int size;
   unsigned char currentClass=1;

   clearImage(outputImage, lowX, highX, lowY, highY);
   for(i = lowY; i <= highY; i++) {
      for(j = lowX; j <= highX; j++) {
         if(mask[i][j]&&(inputImage[i][j]>0)&&(outputImage[i][j]<1)) {
            // outputImage[i][j]=currentClass;
            size = labelSingleObject(inputImage, mask, outputImage, j, i,
                                     currentClass, backFore, lowX, highX,
                                     lowY, highY, connectedness);
            currentClass++;
         }
      }
   }
   return(currentClass-1);
}

void findBoundary(unsigned char** inputImage, unsigned char** backgroundMask, unsigned char** outputImage, struct labelList** boundaryValues, int lowX, int highX, int lowY, int highY)
{
   int i,j,k;
   int newI, newJ;
   int isBoundary;
   int boundValue;
   const int connectedX[] = X_NEIGHBORS;
   const int connectedY[] = Y_NEIGHBORS;
   const int CONNECTEDNESS = 4;

   for(i = lowY; i <= highY; i++) {
      for(j = lowX; j <= highX; j++) {
         isBoundary=0;
         for(k = 0; k < CONNECTEDNESS; k++) {
            newI=i+connectedY[k];
            newJ=j+connectedX[k];
            if ((newJ>=lowX)&&(newJ<=highX)&&(newI>=lowY)&&(newI<=highY)) {
               if ((inputImage[i][j])&&
                   ((inputImage[newI][newJ])==0)&&
                   (backgroundMask[newI][newJ])) {
                  isBoundary=1;
                  if(boundaryValues) {
                     boundValue=backgroundMask[newI][newJ]-1;
                     if(!inList(*boundaryValues, boundValue)) {
                        *boundaryValues=addLabel(*boundaryValues, boundValue);
                     }
                  }
               }
            }
         }
         outputImage[i][j]=isBoundary;
      }
   }
}

// Perform a majority filter on image
void majorityFilter(unsigned char** inputImage, unsigned char** mask, unsigned char** outputImage, int filterRadius, int lowX, int highX, int lowY, int highY)
{
   int i,j,k,l;
   int top, bottom, left, right;
   int count[256];
   int maxCount, maxClass;

   for(i=lowY;i<=highY;i++) {
      for(j=lowX;j<=highX;j++) {
         if(mask[i][j]) {
            for(k=0;k<256;k++) { count[k]=0; }
            top=maxInt(lowY, i-filterRadius);
            bottom=minInt(highY, i+filterRadius);
            left=maxInt(lowX, j-filterRadius);
            right=minInt(highX, j+filterRadius);
            for(k=top;k<=bottom;k++) {
               for(l=left;l<=right;l++) {
                  if(mask[i][j]) {
                     count[inputImage[k][l]]++;
                  }
               }
            }
            maxCount=maxClass=0;
            for(k=1;k<256;k++) {
               if(count[k]>maxCount) {
                  maxCount=count[k];
                  maxClass=k;
               }
            }
            outputImage[i][j]=maxClass;
         }
      }
   }
}
