/*
Copyright (c) 2001 The University of Texas
All Rights Reserved.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

The GNU Public License is available in the file LICENSE, or you
can write to the Free Software Foundation, Inc., 59 Temple Place -
Suite 330, Boston, MA 02111-1307, USA, or you can find it on the
World Wide Web at http://www.fsf.org.

Programmers:    Wade Schwartzkopf
Version:        @(#)labels.c    1.0     02/24/00

The authors are with the Laboratory for Image and Video Engineering
at The University of Texas at Austin, and can be reached at
wade@.ece.utexas.edu.
*/

#include <math.h>
#include <stdlib.h>

#include "labels.h"

struct labelList* addLabel(struct labelList* currentList, int newlabel)
{
   struct labelList* tempptr;

   tempptr = (struct labelList*) malloc(sizeof(struct labelList));
   tempptr->value=newlabel;
   tempptr->next=currentList;
   return tempptr;
}

struct labelList* removeLabel(int label, struct labelList* currentList)
{
   struct labelList* tempptr;
   struct labelList* tempptr2;

   if(!currentList) { return NULL; }
   if((currentList->value)==label) {
      tempptr=currentList->next;
      free(currentList);
      return tempptr;
   }
   for(tempptr = currentList; tempptr; tempptr = tempptr2) {
      tempptr2 = tempptr->next;
      if(tempptr2) {
         if((tempptr2->value)==label) {
            free(tempptr->next);
            tempptr->next=tempptr2->next;
         }
      }
   }
   return currentList;
}

struct labelList* copyLabels(struct labelList* oldList)
{
   struct labelList* tempptr;
   struct labelList* newList=NULL;

   for(tempptr = oldList; tempptr; tempptr = tempptr->next) {
      newList=addLabel(newList, tempptr->value);
   }
   return newList;
}

void freeLabelList(struct labelList* currentList)
{
   struct labelList* tempptr;
   struct labelList* tempptr2;

   for(tempptr = currentList; tempptr; tempptr = tempptr2) {
      tempptr2 = tempptr->next;
      free(tempptr);
   }
}

int inList(struct labelList* listToSearch, int label)
{
   struct labelList* tempptr;

   for(tempptr = listToSearch; tempptr; tempptr = tempptr->next) {
      if(tempptr->value==label) return 1;
   }

   return 0;
}

struct labelList* listLabels(unsigned char** inputImage, int rows, int cols)
{
   int i,j;
   struct labelList* currentList=NULL;

   for(i = 0; i < rows; i++) {
      for(j = 0; j < cols; j++) {
         if(inputImage[i][j]>0) {
            if(!inList(currentList, inputImage[i][j])) {
               currentList=addLabel(currentList, inputImage[i][j]);
            }
         }
      }
   }

   return currentList;
}

void printLabelList(struct labelList* listToPrint)
{
   struct labelList* tempptr;

   for(tempptr = listToPrint; tempptr; tempptr = tempptr->next) {
      printf("Label = %d\r\n", tempptr->value);
   }
}
