/*
Copyright (c) 1998 The University of Texas
All Rights Reserved.
 
This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.
 
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 
The GNU Public License is available in the file LICENSE, or you
can write to the Free Software Foundation, Inc., 59 Temple Place -
Suite 330, Boston, MA 02111-1307, USA, or you can find it on the
World Wide Web at http://www.fsf.org.
 
Programmers:	Wade Schwartzkopf
Version:        @(#)mf_segment.c	1.0	08/16/02


The author performed this work while he was with the Department of
Electrical and Computer Engineering, The University of Texas at Austin,
Austin, TX.  Wade Schwartzkopf can be reached at wade@schwartzkopf.org.
The professors supervising the work were Prof. Alan C. Bovik,
bovik@ece.utexas.edu, and Prof. Brian L. Evans, bevans@ece.utexas.edu.
*/

/*
An example of running this program is:

mfSegment inputImage MFISHdirectory outputImageFilename fileExt 517 645

*/

/* Includes */

#include <stdio.h>

#include "readIntArg.h"
#include "my_matrix_stuff.h"
#include "segmentation.h"

/* Constants */

#define FILESEP '/'

/* Main routine */
int main(int argc, char *argv[])
{
   const int minArgs=1;
   const int maxArgs=6;
   char ASI[6]="DTOGC5";
   char PSI[6]="D36ECI";
   char VYSIS[6]="DFYGAR";
   char* dyeset = ASI;
   int i;
   int imageType = 0;
   int numRows = 0; // 517
   int numColumns = 0; // 645
   unsigned char **chromMask = NULL;
   unsigned char **outputByteImage = NULL;
   char filenames[6][FILENAME_MAX];
   char mfDir[FILENAME_MAX];
   char outputFilename[FILENAME_MAX];
   char imageExt[4] = "png\0";
   char tempstr[FILENAME_MAX];
   MFISH inputMFISH = NULL;
   struct trainDataStruct trainData;

   /* Check for valid arguments */
   if (argc < (minArgs+1)) {
      fprintf(stderr, "You passed %d arguments and at least %d arguments are required.\n", argc - 1, minArgs);
      exit(1);
   }
   else if (argc > maxArgs+1) {
      fprintf(stderr, "You passed too many arguments.  Ignoring extra arguments.\n");
   }
   if (argc >= 3) {
      strncpy(mfDir, argv[2], FILENAME_MAX);
   }
   else {
      strcpy(mfDir, "mfdir");
   }
   if (argc >= 4) {
      strncpy(outputFilename, argv[5], FILENAME_MAX);
   }
   else {
      strcpy(outputFilename, "images/out");
   }
   if (argc >= 5) {
      strncpy(imageExt, argv[4], 4);
   }
   if (argc == 6) {
      numColumns = (numRows = readIntArg("Number of rows", argv[5], 1));
   }
   else if (argc >= 7) {
      numRows = readIntArg("Number of rows", argv[5], 1);
      numColumns = readIntArg("Number of columns", argv[6], 1);
   }

   if(argv[1][0]=='A') {dyeset=ASI;}
   else if (argv[1][0]=='P') {dyeset=PSI;}
   else if (argv[1][0]=='V') {dyeset=VYSIS;}
   else {
      fprintf(stderr, "Invalid dye set.  First character of file name must be A, P, or V.\n");
      exit(1);
   }

   /* Read MFISH images */
   printf("Reading MFISH images...\r\n");
   // Total file name is directory plus filename
   sprintf(tempstr, "%s%c%c%c%c__%c%c%c%s", mfDir, FILESEP,
           argv[1][0], argv[1][1], argv[1][2], argv[1][5], argv[1][6], FILESEP,
           argv[1]);
   for(i=0;i<6;i++) {
      // Create entire path/filename for each image
      sprintf(filenames[i], "%s%c.%s", tempstr, dyeset[i], imageExt);

      /* Read image in 2-D array of bytes */
      // If PGM or PNG format, set numRows and numColumns from header data.
      imageType = readHeader(filenames[i], &numRows, &numColumns);
      if(imageType<0) {
         fprintf(stderr,
                 "File '%s' could not be opened", filenames[i]);
         exit(1);
      }
      if((numRows<=0) || (numColumns<=0)) {
         fprintf(stderr,
                 "File '%s' is not PGM or PNG format.  "
                 "Reading raw image data requires valid arguments "
                 "for image height and width.\n", filenames[i]);
         exit(1);
      }
   }
   inputMFISH = allocateMFISH(filenames[0], filenames[1], filenames[2],
                              filenames[3], filenames[4], filenames[5],
                              numRows, numColumns);
 
   /* Read training data */
   printf("Reading training data...\r\n");
   readTrainingData(&trainData, mfDir, argv[1]);

   /* Read background/foreground separation */
   chromMask=allocateByteMatrix(numRows, numColumns);
   sprintf(tempstr, "%s%c.%s", tempstr, 'K', imageExt);
   readImageAs2DByte(tempstr, chromMask, &numRows, &numColumns);
   threshold(chromMask, chromMask, 0, numRows, numColumns);

   /* Segment image */
   outputByteImage=allocateByteMatrix(numRows, numColumns);
   segment(inputMFISH, chromMask, &trainData, outputByteImage, numRows, numColumns);

   /* Save output image */
   printf("Savings results...\r\n");
   sprintf(tempstr, "%s.%s", outputFilename, imageExt);
   writeImageAs2DByte(tempstr, outputByteImage, numRows, numColumns, imageType);

   freeByteMatrix(outputByteImage);
   freeByteMatrix(chromMask);
   freeTrainingDataStruct(&trainData);
   freeMFISH(inputMFISH);

   return 0;
}
