/*
Copyright (c) 2001 The University of Texas
All Rights Reserved.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

The GNU Public License is available in the file LICENSE, or you
can write to the Free Software Foundation, Inc., 59 Temple Place -
Suite 330, Boston, MA 02111-1307, USA, or you can find it on the
World Wide Web at http://www.fsf.org.

Programmers:    Wade Schwartzkopf
Version:        @(#)readWritePNG.c    1.0     02/24/00

The authors are with the Laboratory for Image and Video Engineering
at The University of Texas at Austin, and can be reached at
wade@.ece.utexas.edu.
*/

#include "readWritePNG.h"

// Check to see if file is PNG format
#define PNG_BYTES_TO_CHECK 8
int FileMatchPNG(char *filename, int *numRowsPtr, int *numColumnsPtr) {
   FILE *fp;
   png_structp png_ptr;
   png_infop info_ptr;
   char buf[PNG_BYTES_TO_CHECK];

   if ((fp = fopen(filename, "rb")) == NULL) { return 0; }
   if (fread(buf, 1, PNG_BYTES_TO_CHECK, fp) != PNG_BYTES_TO_CHECK) {
      return -1;
   }

   // Is this a valid PNG file?
   if (png_sig_cmp(buf, (png_size_t)0, PNG_BYTES_TO_CHECK)) {
      return 0;
   }

   // Create libpng data structures for reading PNG image
   png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING,
                                    NULL, NULL, NULL);
   if (!png_ptr) {
      fclose(fp);
      return -1; }
   info_ptr = png_create_info_struct(png_ptr);
   if (!info_ptr) {
      png_destroy_read_struct(&png_ptr, (png_infopp)NULL,
                              (png_infopp)NULL);
      fclose(fp);
      return -1; }
   
   png_init_io(png_ptr, fp);
   png_set_sig_bytes(png_ptr, PNG_BYTES_TO_CHECK);
   png_read_info(png_ptr, info_ptr);
   // These next two lines screw everything up
   *numRowsPtr = png_get_image_height(png_ptr, info_ptr);
   *numColumnsPtr = png_get_image_width(png_ptr, info_ptr);

   png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp)NULL);
   fclose(fp);

   return 1;
}

int FileReadPNG(char *filename, byte *imgBufferPtr)
{
   FILE *fp;
   png_structp png_ptr;
   png_infop info_ptr;
   png_bytepp imageRowPointers;
   int x, y, imageHeight, imageWidth;
   byte *tmpImgPtr;
   
   fp = fopen(filename, "rb");
   if (!fp) {
      fclose(fp);
      return TCL_ERROR;
   }

   // Create libpng data structures for reading PNG image
   png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING,
                                    NULL, NULL, NULL);
   if (!png_ptr) {
      fclose(fp);
      return TCL_ERROR; }
   info_ptr = png_create_info_struct(png_ptr);
   if (!info_ptr) {
      png_destroy_read_struct(&png_ptr, (png_infopp)NULL,
                              (png_infopp)NULL);
      fclose(fp);
      return TCL_ERROR; }

   png_init_io(png_ptr, fp);
   // Read PNG image into byte-sized pixels
   png_read_png(png_ptr, info_ptr,
                PNG_TRANSFORM_STRIP_16 | PNG_TRANSFORM_PACKING, NULL);
   // Check for non-greyscale images
   if (png_get_color_type(png_ptr, info_ptr) != PNG_COLOR_TYPE_GRAY) {
      fprintf(stderr, "File '%s' is not a greyscale image.  Currently"
                      "only greyscale images are supported.\n", filename);
      return TCL_ERROR; 
   }

   // Convert PNG data structure into raw image data
   imageHeight=png_get_image_height(png_ptr, info_ptr);
   imageWidth=png_get_image_width(png_ptr, info_ptr);
   imageRowPointers=png_get_rows(png_ptr, info_ptr);
   tmpImgPtr=imgBufferPtr;
   for(y=0;y<imageHeight;y++) {
      for(x=0;x<imageWidth;x++) {
         *(tmpImgPtr++)=imageRowPointers[y][x];
      }
   }

   png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp)NULL);
   fclose(fp);

   return TCL_OK;
}

int FileWritePNG(char *filename, byte *imgBufferPtr, int width, int height)
{
   FILE *fp;
   png_structp png_ptr;
   png_infop info_ptr;
   png_bytep row_pointers[height];
   int i;

   /* open the file */
   fp = fopen(filename, "wb");
   if (fp == NULL)
      return (TCL_ERROR);

   png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING,
      NULL, NULL, NULL);
   if (png_ptr == NULL) {
      fclose(fp);
      return (TCL_ERROR);
   }

   info_ptr = png_create_info_struct(png_ptr);
   if (info_ptr == NULL) {
      fclose(fp);
      png_destroy_write_struct(&png_ptr,  (png_infopp)NULL);
      return (TCL_ERROR);
   }

   if (setjmp(png_jmpbuf(png_ptr))) {
      /* If we get here, we had a problem reading the file */
      fclose(fp);
      png_destroy_write_struct(&png_ptr, &info_ptr);
      return (TCL_ERROR);
   }

   png_init_io(png_ptr, fp);

   // Write image here
   png_set_IHDR(png_ptr, info_ptr, width, height, 8, PNG_COLOR_TYPE_GRAY,
      PNG_INTERLACE_NONE, PNG_COMPRESSION_TYPE_BASE, PNG_FILTER_TYPE_BASE);
   png_write_info(png_ptr, info_ptr);

   for (i = 0; i < height; i++)
      row_pointers[i] = imgBufferPtr + i*width;
   png_write_image(png_ptr, row_pointers);
   png_write_end(png_ptr, info_ptr);

   png_destroy_write_struct(&png_ptr, &info_ptr);
   fclose(fp);

   return TCL_OK;
}
